//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "App_Double.h"
#include "Moa_Files.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_double {
//---------------------------------------------------------------------------
void __fastcall TDoubleExecutor::Prepare()
{
	FCreatedDirectory = AnsiString();
}
//---------------------------------------------------------------------------
void __fastcall TDoubleExecutor::Unprepare()
{
	// Do nothing
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDoubleExecutor::AddOriginalName(const AnsiString& ADirectory)
{
	AnsiString LDirectory;
	try {
		AnsiString LOriginalName = ExtractFileName(SourceFile);
		if(RemoveExtension && TRegularFileUtilities::Exists(SourceFile)) {
			LOriginalName = ChangeFileExt(LOriginalName, AnsiString());
		}
		LDirectory =
			IncludeTrailingPathDelimiter(ADirectory) +
			TPathUtilities::Trim(OriginalNamePrefix + LOriginalName + OriginalNameSuffix);
	}
	catch(EFileError& LException) {
		Fail();
	}
	return LDirectory;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDoubleExecutor::AddSerialNumber(const AnsiString& ADirectory)
{
	if(!AvoidNameCollision) {
		return ADirectory;
	}
	if(MinSerialNumber > MaxSerialNumber) {
		throw EAppError("The minimum serial number is exceeding the maximum.");
	}
	AnsiString LDirectory;
	for(int LSerialNumber = MinSerialNumber; LSerialNumber <= MaxSerialNumber; ++LSerialNumber) {
		if(LSerialNumber == MinSerialNumber) {
			LDirectory = ADirectory;
		}
		else {
			switch(SerialNumberPosition) {
			case spBeginning:
				LDirectory =
					ExtractFilePath(ADirectory) +
					TPathUtilities::TrimLeft(SerialNumberPrefix) + LSerialNumber + SerialNumberSuffix +
					ExtractFileName(ADirectory);
				break;
			case spEnd:
				LDirectory =
					ExtractFilePath(ADirectory) +
					ExtractFileName(ADirectory) +
					SerialNumberPrefix + LSerialNumber + TPathUtilities::TrimRight(SerialNumberSuffix);
				break;
			default:
				throw EAppError("The program does not support the position.");
			}
		}
		if(!TFileUtilities::Exists(LDirectory)) {
			break;
		}
		if(LSerialNumber == MaxSerialNumber) {
			Fail();
		}
	}
	return LDirectory;
}
//---------------------------------------------------------------------------
void __fastcall TDoubleExecutor::CreateDirectory(const AnsiString& ADirectory)
{
	if(TFileUtilities::Exists(ADirectory) || ADirectory.Length() == 0 || !ForceDirectories(ADirectory)) {
		Fail();
	}
	FCreatedDirectory = ADirectory;
}
//---------------------------------------------------------------------------
void __fastcall TDoubleExecutor::Fail()
{
	throw EDoubleFail("The program could not create a directory.");
}
//---------------------------------------------------------------------------
__fastcall TDoubleExecutor::TDoubleExecutor() :
	TObject(),
	FSourceFile(),
	FOutputDirectory(),
	FOriginalNamePrefix(),
	FOriginalNameSuffix(),
	FRemoveExtension(true),
	FAvoidNameCollision(true),
	FSerialNumberPrefix(),
	FSerialNumberSuffix(),
	FMinSerialNumber(1),
	FMaxSerialNumber(100),
	FSerialNumberPosition(spEnd),
	FCreatedDirectory()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TDoubleExecutor::~TDoubleExecutor()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TDoubleExecutor::Execute()
{
	Prepare();
	try {
		CreateDirectory(AddSerialNumber(AddOriginalName(OutputDirectory)));
	}
	catch(...) {
		Unprepare();
		throw;
	}
	Unprepare();
}
//---------------------------------------------------------------------------
} // End namespace App_double
//---------------------------------------------------------------------------

