//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "App_DoubleEntries.h"
#include "Moa_Files.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_doubleentries {
//---------------------------------------------------------------------------
AnsiString __fastcall TDoubleEntry::EnsureValidPath(const AnsiString& APath)
{
	AnsiString LPath = APath;
	try {
		if(!TPathUtilities::IsAbsolute(LPath)) {
			throw EAppError(AnsiString().sprintf(
				"The program cannot accept the path \"%d\" because it is not in absolute form.",
				APath.c_str()));
		}
		LPath = ExpandFileName(LPath);
		LPath = TFileUtilities::GetLongPath(LPath);

		// Trap the path with an empty name - root directories match
		if(ExtractFileName(LPath).Length() == 0) {
			throw EAppError(AnsiString().sprintf(
				"The program cannot accept the path \"%s\" because its name is empty.",
				LPath.c_str()));
		}
	}
	catch(EFileError& LException) {
		throw EAppError(AnsiString().sprintf(
			"The program cannot accept the path \"%s\" because its long form is unavailable.",
			LPath.c_str()));
	}
	return LPath;
}
//---------------------------------------------------------------------------
__fastcall TDoubleEntry::TDoubleEntry(const AnsiString& AFile) :
	TObject(),
	FPath(EnsureValidPath(AFile))
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TDoubleEntry::~TDoubleEntry()
{
	// Do nothing
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
int __fastcall TDoubleEntriesModel::SortCompare(void* AItem1, void* AItem2)
{
	return AnsiCompareStr(
		static_cast<TDoubleEntry*>(AItem1)->Path,
		static_cast<TDoubleEntry*>(AItem2)->Path);
}
//---------------------------------------------------------------------------
void __fastcall TDoubleEntriesModel::Initialize()
{
	try {
		FList = new TObjectList();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TDoubleEntriesModel::Uninitialize()
{
	delete FList;
	FList = NULL;
}
//---------------------------------------------------------------------------
TDoubleEntry* __fastcall TDoubleEntriesModel::GetItem(int AIndex)
{
	return static_cast<TDoubleEntry*>(List->Items[AIndex]);
}
//---------------------------------------------------------------------------
int __fastcall TDoubleEntriesModel::GetCount()
{
	return List->Count;
}
//---------------------------------------------------------------------------
int __fastcall TDoubleEntriesModel::GetCapacity()
{
	return List->Capacity;
}
//---------------------------------------------------------------------------
void __fastcall TDoubleEntriesModel::SetCapacity(int ACapacity)
{
	List->Capacity = ACapacity;
}
//---------------------------------------------------------------------------
void __fastcall TDoubleEntriesModel::DoChange()
{
	if(OnChange != NULL) {
		OnChange(this);
	}
}
//---------------------------------------------------------------------------
__fastcall TDoubleEntriesModel::TDoubleEntriesModel() :
	TModel(),
	FList(NULL),
	FOnChange(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TDoubleEntriesModel::~TDoubleEntriesModel()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TDoubleEntriesModel::AddItems(const DynamicArray<TDoubleEntry*>& AItems)
{
	if(List->Capacity < List->Count + AItems.Length) {
		List->Capacity = List->Count + AItems.Length;
	}
	for(int LI = 0; LI < AItems.Length; ++LI) {
		List->Add(AItems[LI]);
	}
	DoChange();
}
//---------------------------------------------------------------------------
void __fastcall TDoubleEntriesModel::Clear()
{
	List->Clear();
	DoChange();
}
//---------------------------------------------------------------------------
void __fastcall TDoubleEntriesModel::Sort()
{
	List->Sort(&SortCompare);
	DoChange();
}
//---------------------------------------------------------------------------
} // End namespace App_doubleentries
//---------------------------------------------------------------------------

