//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <IniFiles.hpp>
#include "App_DoubleEntries.h"
#include "App_DoubleOptions.h"
#include "App_ErrorDialog.h"
#include "App_Main.h"
#include "App_MenuDialog.h"
#include "App_OptionsDialog.h"
#include "Moa_AutoPointers.h"
#include "Moa_ShellFiles.h"
#include "Moa_Strings.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_main {
//---------------------------------------------------------------------------
void __fastcall TMainPresenter::Initialize()
{
	try {
		FDoubleEntriesModel = new TDoubleEntriesModel();
		FDoubleOptionsModel = new TDoubleOptionsModel();
		LoadSettings();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainPresenter::Uninitialize()
{
	delete FDoubleOptionsModel;
	FDoubleOptionsModel = NULL;

	delete FDoubleEntriesModel;
	FDoubleEntriesModel = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TMainPresenter::LoadSettings()
{
	TAutoPointer<TMemIniFile> LSettings = new TMemIniFile(TCommon::SettingsFile);
	AnsiString LSection = "System";
	{
		SetCurrentDir(TAnsiStringUtilities::Dequote(LSettings->ReadString(
			LSection,
			"CurrentDirectory",
			TAnsiStringUtilities::Quote(GetCurrentDir()))));
	}
	LSection = "Options";
	{
		TDoubleOptions LValue = DoubleOptionsModel->Value;
		LValue.OutputDirectory = static_cast<TOutputDirectoryType>(LSettings->ReadInteger(
			LSection,
			"OutputDirectory",
			LValue.OutputDirectory));
		LValue.OtherOutputDirectory = TAnsiStringUtilities::Dequote(LSettings->ReadString(
			LSection,
			"OtherOutputDirectory",
			TAnsiStringUtilities::Quote(LValue.OtherOutputDirectory)));
		LValue.OriginalNamePrefix = TAnsiStringUtilities::Dequote(LSettings->ReadString(
			LSection,
			"OriginalNamePrefix",
			TAnsiStringUtilities::Quote(LValue.OriginalNamePrefix)));
		LValue.OriginalNameSuffix = TAnsiStringUtilities::Dequote(LSettings->ReadString(
			LSection,
			"OriginalNameSuffix",
			TAnsiStringUtilities::Quote(LValue.OriginalNameSuffix)));
		LValue.RemoveExtension = LSettings->ReadBool(
			LSection,
			"RemoveExtension",
			LValue.RemoveExtension);
		LValue.AvoidNameCollision = LSettings->ReadBool(
			LSection,
			"AvoidNameCollision",
			LValue.AvoidNameCollision);
		LValue.SerialNumberPrefix = TAnsiStringUtilities::Dequote(LSettings->ReadString(
			LSection,
			"SerialNumberPrefix",
			TAnsiStringUtilities::Quote(LValue.SerialNumberPrefix)));
		LValue.SerialNumberSuffix = TAnsiStringUtilities::Dequote(LSettings->ReadString(
			LSection,
			"SerialNumberSuffix",
			TAnsiStringUtilities::Quote(LValue.SerialNumberSuffix)));
		LValue.MinSerialNumber = LSettings->ReadInteger(
			LSection,
			"MinSerialNumber",
			LValue.MinSerialNumber);
		LValue.MaxSerialNumber = LSettings->ReadInteger(
			LSection,
			"MaxSerialNumber",
			LValue.MaxSerialNumber);
		LValue.SerialNumberPosition = static_cast<TSerialNumberPosition>(LSettings->ReadInteger(
			LSection,
			"SerialNumberPosition",
			LValue.SerialNumberPosition));
		LValue.SortEntries = LSettings->ReadBool(
			LSection,
			"SortEntries",
			LValue.SortEntries);
		DoubleOptionsModel->Value = LValue;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainPresenter::SaveSettings()
{
	TAutoPointer<TMemIniFile> LSettings = new TMemIniFile(TCommon::SettingsFile);
	AnsiString LSection = "System";
	{
		LSettings->WriteString(
			LSection,
			"CurrentDirectory",
			TAnsiStringUtilities::Quote(GetCurrentDir()));
	}
	LSection = "Options";
	{
		TDoubleOptions LValue = DoubleOptionsModel->Value;
		LSettings->WriteInteger(
			LSection,
			"OutputDirectory",
			LValue.OutputDirectory);
		LSettings->WriteString(
			LSection,
			"OtherOutputDirectory",
			TAnsiStringUtilities::Quote(LValue.OtherOutputDirectory));
		LSettings->WriteString(
			LSection,
			"OriginalNamePrefix",
			TAnsiStringUtilities::Quote(LValue.OriginalNamePrefix));
		LSettings->WriteString(
			LSection,
			"OriginalNameSuffix",
			TAnsiStringUtilities::Quote(LValue.OriginalNameSuffix));
		LSettings->WriteBool(
			LSection,
			"RemoveExtension",
			LValue.RemoveExtension);
		LSettings->WriteBool(
			LSection,
			"AvoidNameCollision",
			LValue.AvoidNameCollision);
		LSettings->WriteString(
			LSection,
			"SerialNumberPrefix",
			TAnsiStringUtilities::Quote(LValue.SerialNumberPrefix));
		LSettings->WriteString(
			LSection,
			"SerialNumberSuffix",
			TAnsiStringUtilities::Quote(LValue.SerialNumberSuffix));
		LSettings->WriteInteger(
			LSection,
			"MinSerialNumber",
			LValue.MinSerialNumber);
		LSettings->WriteInteger(
			LSection,
			"MaxSerialNumber",
			LValue.MaxSerialNumber);
		LSettings->WriteInteger(
			LSection,
			"SerialNumberPosition",
			LValue.SerialNumberPosition);
		LSettings->WriteBool(
			LSection,
			"SortEntries",
			LValue.SortEntries);
	}
	LSettings->UpdateFile();
}
//---------------------------------------------------------------------------
void __fastcall TMainPresenter::HandleCommandLineParameters()
{
	if(ParamCount() == 0) {
		return;
	}
	DynamicArray<TDoubleEntry*> LItems;
	try {
		LItems.Length = ParamCount();
		for(int LI = 0; LI < LItems.Length; ++LI) {
			LItems[LI] = NULL;
		}
		for(int LI = 0; LI < LItems.Length; ++LI) {
			TAutoPointer<TDoubleEntry> LItem = new TDoubleEntry(ParamStr(LI + 1));
			LItems[LI] = LItem;
			LItem.Release();
		}
		DoubleEntriesModel->AddItems(LItems);
		if(DoubleOptionsModel->Value.SortEntries) {
			DoubleEntriesModel->Sort();
		}
	}
	catch(...) {
		for(int LI = 0; LI < LItems.Length; ++LI) {
			delete LItems[LI];
		}
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainPresenter::CreateDirectories()
{
	TAutoPointer<TStrings> LFailureFiles = new TStringList();
	LFailureFiles->Capacity = DoubleEntriesModel->Count;
	{
		TAutoPointer<TDoubleExecutor> LExecutor = new TDoubleExecutor();
		switch(DoubleOptionsModel->Value.OutputDirectory) {
		case odAtSource:
			// Do nothing
			break;
		case odDesktop:
			LExecutor->OutputDirectory = TSpecialShellFiles::GetDesktop();
			break;
		case odAsk:
			{
				AnsiString LDirectory = GetCurrentDir();
				if(!SelectDirectory("Choose the output directory.", WideString(), LDirectory)) {
					Abort();
				}
				SetCurrentDir(LDirectory);
				LExecutor->OutputDirectory = LDirectory;
			}
			break;
		case odOther:
			LExecutor->OutputDirectory = DoubleOptionsModel->Value.OtherOutputDirectory;
			break;
		default:
			throw EAppError("The program does not support the position.");
		}
		LExecutor->OriginalNamePrefix = DoubleOptionsModel->Value.OriginalNamePrefix;
		LExecutor->OriginalNameSuffix = DoubleOptionsModel->Value.OriginalNameSuffix;
		LExecutor->RemoveExtension = DoubleOptionsModel->Value.RemoveExtension;
		LExecutor->AvoidNameCollision = DoubleOptionsModel->Value.AvoidNameCollision;
		LExecutor->SerialNumberPrefix = DoubleOptionsModel->Value.SerialNumberPrefix;
		LExecutor->SerialNumberSuffix = DoubleOptionsModel->Value.SerialNumberSuffix;
		LExecutor->MinSerialNumber = DoubleOptionsModel->Value.MinSerialNumber;
		LExecutor->MaxSerialNumber = DoubleOptionsModel->Value.MaxSerialNumber;
		LExecutor->SerialNumberPosition = DoubleOptionsModel->Value.SerialNumberPosition;
		for(int LI = 0; LI < DoubleEntriesModel->Count; ++LI) {
			try {
				LExecutor->SourceFile = DoubleEntriesModel->Items[LI]->Path;
				if(DoubleOptionsModel->Value.OutputDirectory == odAtSource) {
					LExecutor->OutputDirectory = ExtractFileDir(DoubleEntriesModel->Items[LI]->Path);
				}
				LExecutor->Execute();
			}
			catch(EDoubleFail& LException) {
				LFailureFiles->Add(DoubleEntriesModel->Items[LI]->Path);
			}
		}
	}
	if(LFailureFiles->Count > 0) {
		TErrorDialog::EnsureCreated();
		ErrorDialog->Values->Assign(LFailureFiles);
		ErrorDialog->Execute();
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainPresenter::ShowOptions()
{
	TOptionsDialog::EnsureCreated();
	OptionsDialog->Values = DoubleOptionsModel->Value;
	if(!OptionsDialog->Execute()) {
		Abort();
	}
	DoubleOptionsModel->Value = OptionsDialog->Values;
}
//---------------------------------------------------------------------------
void __fastcall TMainPresenter::OpenHelp()
{
	SHELLEXECUTEINFO LExecuteInfo = {0};
	LExecuteInfo.cbSize = sizeof(SHELLEXECUTEINFO);
	LExecuteInfo.fMask = SEE_MASK_FLAG_NO_UI;
	LExecuteInfo.lpVerb = "open";
	LExecuteInfo.lpFile = TCommon::HelpFile.c_str();
	LExecuteInfo.nShow = SW_SHOWNORMAL;
	if(!ShellExecuteEx(&LExecuteInfo)) {
		throw EAppError("The program could not open the help.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainPresenter::ShowAbout()
{
	Application->MessageBox(
		TCommon::About.c_str(),
		"About",
		MB_OK | MB_SETFOREGROUND);
}
//---------------------------------------------------------------------------
__fastcall TMainPresenter::TMainPresenter() :
	TPresenter(),
	FDoubleEntriesModel(NULL),
	FDoubleOptionsModel(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TMainPresenter::~TMainPresenter()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TMainPresenter::Start()
{
	TCursor LCursor = Screen->Cursor;
	Screen->Cursor = crHourGlass;
	try {
		try {
			HandleCommandLineParameters();
			if(DoubleEntriesModel->Count > 0) {
				CreateDirectories();
			}
			else {
				if(!TMenuDialog::EnsureCreated()->Execute()) {
					Abort();
				}
				switch(MenuDialog->Value) {
				case mnOptions:
					ShowOptions();
					break;
				case mnHelp:
					OpenHelp();
					break;
				case mnAbout:
					ShowAbout();
					break;
				default:
					throw EAppError("The program does not support the result.");
				}
			}
		}
		catch(EAbort& LException) {
			// Do nothing
		}
		SaveSettings();
	}
	catch(...) {
		Screen->Cursor = LCursor;
		throw;
	}
	Screen->Cursor = LCursor;
}
//---------------------------------------------------------------------------
} // End namespace App_main
//---------------------------------------------------------------------------

