//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "App_Processes.h"
#include "Moa_AutoPointers.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_processes {
//---------------------------------------------------------------------------
__fastcall TProcessCreator::TProcessCreator() :
	TObject(),
	FFile(),
	FParameters(),
	FWorkingDirectory(),
	FShowCommand(SW_NORMAL),
	FReturnProcess(true)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TProcessCreator::~TProcessCreator()
{
	// Do nothing
}
//---------------------------------------------------------------------------
TProcess* __fastcall TProcessCreator::Execute()
{
	HANDLE LHandle = NULL;
	{
		SHELLEXECUTEINFO LExecuteInfo = {0};
		LExecuteInfo.cbSize = sizeof(SHELLEXECUTEINFO);
		LExecuteInfo.fMask = SEE_MASK_FLAG_NO_UI;
		if(ReturnProcess) {
			LExecuteInfo.fMask |= SEE_MASK_NOCLOSEPROCESS;
		}
		LExecuteInfo.lpFile = FFile.c_str();
		LExecuteInfo.lpParameters = static_cast<LPCSTR>(FParameters.data());
		LExecuteInfo.lpDirectory = static_cast<LPCSTR>(FWorkingDirectory.data());
		LExecuteInfo.nShow = ShowCommand;
		if(!::ShellExecuteEx(&LExecuteInfo)) {
			throw EProcessCreateError("The program could not create the process.");
		}
		if(LExecuteInfo.hProcess == NULL || !ReturnProcess) {
			throw EProcessUnavailable("The program could not get the handle of the process.");
		}
		LHandle = LExecuteInfo.hProcess;
	}
	TAutoPointer<TProcess> LProcess;
	try {
		LProcess = new TProcess(LHandle);
	}
	catch(...) {
		::CloseHandle(LHandle);
		throw;
	}
	return LProcess.Release();
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TProcess::Initialize()
{
	try {
		// Do nothing
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TProcess::Uninitialize()
{
	::CloseHandle(FHandle);
	FHandle = NULL;
}
//---------------------------------------------------------------------------
__fastcall TProcess::TProcess(HANDLE AHandle) :
	TObject(),
	FHandle(AHandle)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TProcess::~TProcess()
{
	// Do nothing
}
//---------------------------------------------------------------------------
bool __fastcall TProcess::IsTerminated(int ATimeoutInterval)
{
	switch(::WaitForSingleObject(Handle, ATimeoutInterval < 0 ? INFINITE : ATimeoutInterval)) {
	case WAIT_ABANDONED:
	case WAIT_OBJECT_0:
		return true;
	case WAIT_TIMEOUT:
		return false;
	case WAIT_FAILED:
	default:
		throw EProcessWaitFail("The program cannot determine whether the process has terminated.");
	}
}
//---------------------------------------------------------------------------
} // End namespace App_processes
//---------------------------------------------------------------------------

