//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "App_ProgramDialog.h"
#include "Moa_AutoPointers.h"
#include "Moa_Files.h"
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
TProgramDialog* ProgramDialog = NULL;
//---------------------------------------------------------------------------
void __fastcall TProgramDialog::Button1Click(TObject* ASender)
{
	OpenDialog1->FileName = Input2;
	if(!OpenDialog1->Execute()) {
		Abort();
	}
	Input2 = OpenDialog1->FileName;
}
//---------------------------------------------------------------------------
void __fastcall TProgramDialog::Button2Click(TObject* ASender)
{
	AnsiString LDirectory = Input5;
	if(!SelectDirectory("Choose the working directory.", WideString(), LDirectory)) {
		Abort();
	}
	SetCurrentDir(LDirectory);
	Input5 = LDirectory;
}
//---------------------------------------------------------------------------
void __fastcall TProgramDialog::ComboBox1Exit(TObject* ASender)
{
	Input6;
}
//---------------------------------------------------------------------------
void __fastcall TProgramDialog::Edit1Exit(TObject* ASender)
{
	Input1;
}
//---------------------------------------------------------------------------
void __fastcall TProgramDialog::Edit2Exit(TObject* ASender)
{
	Input2;
}
//---------------------------------------------------------------------------
void __fastcall TProgramDialog::FormCloseQuery(TObject* ASender, bool& ACanClose)
{
	if(ModalResult == mrOk) {
		UpdateValues();
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgramDialog::HandleInvalidValue(const AnsiString& AMessage)
{
	throw EAppError(AMessage);
}
//---------------------------------------------------------------------------
void __fastcall TProgramDialog::HandleInvalidInput(const AnsiString& AMessage, TWinControl* AControl)
{
	ActiveControl = AControl;
	throw EAppError(AMessage);
}
//---------------------------------------------------------------------------
void __fastcall TProgramDialog::Initialize()
{
	try {
		FValues = new TProgramEntry();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgramDialog::Uninitialize()
{
	delete FValues;
	FValues = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TProgramDialog::UpdateInputs()
{
	Input1 = Values->DisplayName;
	Input2 = Values->File;
	Input3 = Values->DocumentPrefix;
	Input4 = Values->DocumentSuffix;
	Input5 = Values->WorkingDirectory;
	Input6 = Values->ShowCommand;
	Input7 = Values->WaitForTermination;
}
//---------------------------------------------------------------------------
void __fastcall TProgramDialog::UpdateValues()
{
	TAutoPointer<TProgramEntry> LInputs = new TProgramEntry();
	LInputs->DisplayName = Input1;
	LInputs->File = Input2;
	LInputs->DocumentPrefix = Input3;
	LInputs->DocumentSuffix = Input4;
	LInputs->WorkingDirectory = Input5;
	LInputs->ShowCommand = Input6;
	LInputs->WaitForTermination = Input7;
	Values->Assign(LInputs);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TProgramDialog::GetInput1()
{
	AnsiString LInput = Edit1->Text;
	if(LInput.Length() == 0) {
		HandleInvalidInput("Type a display name.", Edit1);
	}
	return LInput;
}
//---------------------------------------------------------------------------
void __fastcall TProgramDialog::SetInput1(const AnsiString& AInput)
{
	Edit1->Text = AInput;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TProgramDialog::GetInput2()
{
	AnsiString LInput = Edit2->Text;
	if(!TFileUtilities::Exists(LInput)) {
		HandleInvalidInput("Type the path of an existent file.", Edit2);
	}
	return LInput;
}
//---------------------------------------------------------------------------
void __fastcall TProgramDialog::SetInput2(const AnsiString& AInput)
{
	Edit2->Text = AInput;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TProgramDialog::GetInput3()
{
	return Edit3->Text;
}
//---------------------------------------------------------------------------
void __fastcall TProgramDialog::SetInput3(const AnsiString& AInput)
{
	Edit3->Text = AInput;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TProgramDialog::GetInput4()
{
	return Edit4->Text;
}
//---------------------------------------------------------------------------
void __fastcall TProgramDialog::SetInput4(const AnsiString& AInput)
{
	Edit4->Text = AInput;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TProgramDialog::GetInput5()
{
	return Edit5->Text;
}
//---------------------------------------------------------------------------
void __fastcall TProgramDialog::SetInput5(const AnsiString& AInput)
{
	Edit5->Text = AInput;
}
//---------------------------------------------------------------------------
int __fastcall TProgramDialog::GetInput6()
{
	int LInput = 0;
	switch(ComboBox1->ItemIndex) {
	case 0:
		LInput = SW_SHOWNORMAL;
		break;
	case 1:
		LInput = SW_MINIMIZE;
		break;
	case 2:
		LInput = SW_MAXIMIZE;
		break;
	default:
		HandleInvalidInput("Choose an item.", ComboBox1);
	}
	return LInput;
}
//---------------------------------------------------------------------------
void __fastcall TProgramDialog::SetInput6(int AInput)
{
	int LItemIndex = 0;
	switch(AInput) {
	case SW_SHOWNORMAL:
		LItemIndex = 0;
		break;
	case SW_MINIMIZE:
		LItemIndex = 1;
		break;
	case SW_MAXIMIZE:
		LItemIndex = 2;
		break;
	default:
		HandleInvalidValue("The program does not support the window state.");
	}
	ComboBox1->ItemIndex = LItemIndex;
}
//---------------------------------------------------------------------------
bool __fastcall TProgramDialog::GetInput7()
{
	return CheckBox1->Checked;
}
//---------------------------------------------------------------------------
void __fastcall TProgramDialog::SetInput7(bool AInput)
{
	CheckBox1->Checked = AInput;
}
//---------------------------------------------------------------------------
__fastcall TProgramDialog::TProgramDialog(TComponent* AOwner) :
	TForm(AOwner),
	FValues(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TProgramDialog::~TProgramDialog()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
TProgramDialog* __fastcall TProgramDialog::EnsureCreated()
{
	if(ProgramDialog == NULL) {
		ProgramDialog = new TProgramDialog(Application);
	}
	return ProgramDialog;
}
//---------------------------------------------------------------------------
bool __fastcall TProgramDialog::Execute()
{
	UpdateInputs();
	ActiveControl = NULL;
	SelectFirst();
	return ShowModal() == mrOk;
}
//---------------------------------------------------------------------------
void __fastcall TProgramDialog::Reset()
{
	TAutoPointer<TProgramEntry> LDefaultValues = new TProgramEntry();
	LDefaultValues->DisplayName = "Untitled";
	LDefaultValues->File = Application->ExeName;
	LDefaultValues->DocumentPrefix = "\"";
	LDefaultValues->DocumentSuffix = "\"";
	LDefaultValues->WorkingDirectory = AnsiString();
	LDefaultValues->ShowCommand = SW_SHOWNORMAL;
	LDefaultValues->WaitForTermination = true;
	Values->Assign(LDefaultValues);
}
//---------------------------------------------------------------------------

